import sys
from git import Repo as GITRepo
from dvc_cc.hyperopt.variable import *
import subprocess
import uuid
import os
from dvc_cc.bcolors import *

def get_main_git_directory_path():
    gitrepo = GITRepo('.')
    git_path = gitrepo.common_dir.split('/.git')[0]
    return git_path


DESCRIPTION = 'Use this to define your pipeline with or without hyperparameters. It is the same use as '\
              +bcolors.OKBLUE+'dvc run --no-exec'+bcolors.ENDC + '.. If you want use ' \
               'hyperparameters you can set use i.e. {{a}} for the hyperparameter a. The script will ask you ' \
               'for the type of the hyperparameter. You can also set the type directly with i.e. {{a:int}}. ' \
               'You should use " for your command!\n' \
               'I.E.: '+bcolors.OKBLUE+'dvc-cc hyperopt new -d data.npy -o tensorboard -m summary.json "python train.py --learning-rate {{lr}} --batch-size {{bs}}"'+bcolors.ENDC


def main():

    # go to the main git directory.
    os.chdir(get_main_git_directory_path())

    if not os.path.exists('dvc'):
        os.mkdir('dvc')
    
    if not os.path.exists('dvc/.hyperopt'):
        os.mkdir('dvc/.hyperopt')

    if len(sys.argv) == 1:
        print(DESCRIPTION)
        exit(0)

    # get all existent variables
    vc = VariableCache()
    hyperopt_files = [f for f in os.listdir('dvc/.hyperopt') if f.endswith('.hyperopt')]
    for f in hyperopt_files:
        vc.register_dvccc_file('dvc/.hyperopt/'+f)

    # create the dvc file
    found_user_filename = False
    command_start_in = 0
    for i in range(len(sys.argv)-1):
        if sys.argv[i] == '-f':
            found_user_filename = True
            sys.argv[i + 1]='dvc/'+sys.argv[i + 1].replace('/', '_')
            output_filename = sys.argv[i+1]


    if found_user_filename:
        subprocess.call(['dvc', 'run', '--no-exec'] + sys.argv[1:])
    else:
        output_filename = 'dvc/'+ str(uuid.uuid4())+'.dvc'
        subprocess.call(['dvc', 'run', '--no-exec', '-f', output_filename] + sys.argv[1:])

    new_hyperopt_filename = 'dvc/.hyperopt/'+output_filename[4:-4]+'.hyperopt'
    os.rename(output_filename, new_hyperopt_filename)

    try:
        vc.register_dvccc_file(new_hyperopt_filename)
    except KeyboardInterrupt as ki:
        # Delete the file if the user want to cancel the job
        os.remove(new_hyperopt_filename)
        raise ki
    vc.update_dvccc_files()

    subprocess.call(['git', 'add', 'dvc/.hyperopt/*'])



























#!/usr/bin/env python3

import os
from argparse import ArgumentParser
import subprocess
import shutil
from dvc_cc.bcolors import *
DESCRIPTION = 'With this script you can mount or unmount the live output directory.'

def get_gitinformation():
    # TODO: use the intern python-git for this.
    out = subprocess.check_output(["git", "config", "--get", "remote.origin.url"]).decode("utf8")
    if out.startswith('https://'):
        _,_, gitrepo,gitowner,gitname = out.split('/')
    else:
        gitrepo = out[4:out.find(':')]
        gitowner = out[out.find(':')+1:out.find('/')]
        gitname = out[out.find('/')+1:]

    gitname = gitname[:gitname.find('.git')+4]
    return gitrepo,gitowner,gitname


def get_dvcurl():
    dvc_url = []
    try:
      with open(".dvc/config.local","r") as fi:
        for ln in fi:
            if ln.startswith("url = ") or ln.startswith("url="):
              dvc_url.append(ln)
    except:
        print('No .dvc/config.local was found.')
        try:
          with open(".dvc/config","r") as fi:
            for ln in fi:
                if ln.startswith("url =") or ln.startswith("url="):
                  dvc_url.append(ln)
        except:
          print('No .dvc/config was found.')

    if len(dvc_url) != 1:
        if len(dvc_url) == 0:
            print(bcolors.WARNING+'Warning: no url was found. please set the url in the .dvc/config or .dvc/config.local file.'+ bcolors.ENDC)
        if len(dvc_url) > 1:
            print(bcolors.WARNING+'Warning: Multiple url was found. only one url is currently allowed'+bcolors.ENDC)
        print('\tPlease specifier the servername and the repository.')
        dvc_server = input("\tdvc_servername: ")
        dvc_path = input("\tdvc_path_to_working_repository: ")
    return dvc_url[0].split('=')[1].replace(' ', '').replace('\n', '').split('://')[1]

def main():
    parser = ArgumentParser(description=DESCRIPTION)

    parser.add_argument('-um', '--umount', dest='umount', action='store_true', default=False,
                        help='If this parameter is set, the live output dir will be umount else it will be mount.')
    args = parser.parse_args()

    if not args.umount:
        if not os.path.exists('tmp_live_output'):
            os.mkdir('tmp_live_output')
            _, git_owner, git_name = get_gitinformation()
            dvc_url = get_dvcurl()
            command = 'sshfs ' + dvc_url + '/' +  git_owner + '/' + git_name + ' tmp_live_output'
            with open('.gitignore', "a+") as f:
                f.write('\ntmp_live_output')
            print(command)
            subprocess.call(command.split(' '))
        else:
            print(bcolors.WARNING+'Warning: tmp_live_output already exists. Use --umount to umount and delete this directory and try again.'+bcolors.ENDC)
    elif args.umount:
        if os.path.exists('tmp_live_output'):
            try:
                subprocess.call(['fusermount','-u','tmp_live_output'])
            finally:
                shutil.rmtree('tmp_live_output')
        else:
            print(bcolors.WARNING+'Warning: tmp_live_output does not exists.'+bcolors.ENDC)



import sys
import os
from dvc_cc.bcolors import *
import subprocess
from subprocess import check_output

SCRIPT_NAME = 'dvc-cc dvc'
TITLE = 'tools'
DESCRIPTION = 'With this script you can call everything what you can do with dvc. For example you could be interested ' \
              'in calling '+ bcolors.OKBLUE+'dvc pipeline show --ascii dvc/train.dvc --commands'+ bcolors.ENDC + ', ' \
              'for visualizing the pipeline. For more information visit https://dvc.org/doc/get-started/visualize.' \
              'This script converts the dvc-cc hyperopt-files to dvc files, ' \
              'call the command, and reconvert the scripts back to hyperopt-files.'



def main():
    argv = sys.argv[1:]
    if '-h' in argv or '--help' in argv or len(argv) == 0:
        print(DESCRIPTION)
    else:
        try:
            #####################################
            # Rename the hyperopt-files to .dvc #
            #####################################
            if os.path.exists('dvc') and os.path.exists('dvc/.hyperopt'):
                list_of_hyperopt_files = [f for f in os.listdir('dvc/.hyperopt') if f.endswith('.hyperopt')]
                for f in list_of_hyperopt_files:
                    os.rename('dvc/.hyperopt/' + f, 'dvc/' + f[:-9] + '.dvc')
            else:
                list_of_hyperopt_files = []

            subprocess.call(sys.argv[1:])

        finally:
            #############################
            # Rename the hyperopt-files #
            #############################
            for f in list_of_hyperopt_files:
                if os.path.exists('dvc/'+f[:-9]+'.dvc'):
                    os.rename('dvc/'+f[:-9]+'.dvc', 'dvc/.hyperopt/' + f)
                else:
                    print(bcolors.WARNING+'Warning: File ' + 'dvc/'+f[:-9]+'.dvc' + ' not found.'+bcolors.ENDC)


# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['dvc_cc',
 'dvc_cc.cancel',
 'dvc_cc.dvc',
 'dvc_cc.git',
 'dvc_cc.hyperopt',
 'dvc_cc.init',
 'dvc_cc.live_output',
 'dvc_cc.output_to_tmp',
 'dvc_cc.output_to_tmp_fast',
 'dvc_cc.run',
 'dvc_cc.run_all_defined',
 'dvc_cc.setting',
 'dvc_cc.status']

package_data = \
{'': ['*']}

install_requires = \
['cc-faice>=8.0,<9.0',
 'dvc>=0,<1',
 'keyring>=19.0,<20.0',
 'matplotlib>=3.1,<4.0',
 'nbconvert>=5.5,<6.0',
 'numpy>=1.16,<2.0',
 'pandas>=0,<1',
 'paramiko>=2.4,<3.0',
 'pyrsistent>=0.15.2,<0.16.0',
 'python-gitlab>=1.8,<2.0',
 'pyyaml>=5.1,<6.0',
 'seaborn>=0,<1']

entry_points = \
{'console_scripts': ['dvc-cc = dvc_cc.main:main']}

setup_kwargs = {
    'name': 'dvc-cc',
    'version': '0.8.6',
    'description': 'This connector is used to combine the work of CC (www.curious-containers.cc) and DVC (Open-source Version Control System for Machine Learning Projects).',
    'long_description': '# The DVC-CC client\nIn this subrepository the [DVC-CC](https://github.com/deep-projects/dvc-cc) client is defined. The client is needed to define hyper optimization and push your jobs to [Curious Containers CC](https://www.curious-containers.cc/).\n\n## Get started\nTODO\n\n### Overview\nTODO\n\n### Tutorials\n- [An old tutorial](https://github.com/deep-projects/dvc-cc/blob/master/dvc-cc/tutorial/SimpleStart.md)\n\n\n## Acknowledgements\nThe DVC-CC software is developed at CBMI (HTW Berlin - University of Applied Sciences). The work is supported by the\nGerman Federal Ministry of Education and Research (project deep.TEACHING, grant number 01IS17056 and project\ndeep.HEALTH, grant number 13FH770IX6).',
    'author': 'Jonas Annuscheit',
    'author_email': 'annusch@htw-berlin.de',
    'url': 'https://github.com/mastaer/dvc-cc.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3

from argparse import ArgumentParser
from dvc.repo import Repo as DVCRepo
from git import Repo as GITRepo
import yaml
import os
from subprocess import check_output
import configparser

import numpy as np
import subprocess

DESCRIPTION = 'This script starts the jobs that are already defined with "dvc-cc run" but run with the parameter "--no-exec".'

def get_main_git_directory_path():
    gitrepo = GITRepo('.')
    git_path = gitrepo.common_dir.split('/.git')[0]
    return git_path

def main():
    parser = ArgumentParser(description=DESCRIPTION)
    args = parser.parse_args()
    
    project_dir = get_main_git_directory_path()

    os.chdir(project_dir)
    
    gitrepo = GITRepo('.')
    dvcrepo = DVCRepo('.')

    subprocess.call(['git', 'push'])
    subprocess.call(['git', 'push', 'origin', '--tags'])

    if os.path.exists('.dvc_cc/cc_agency_experiments.yml'):
        with open(".dvc_cc/cc_agency_experiments.yml", 'r') as stream:
            try:
                experiments = yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                print(exc)
                exit(1)

        start_an_experiment = False
        for k in experiments.keys():
            # find all files
            paths = []
            if experiments[k]['id'] is None:
                print('Start job ' + k)
                start_an_experiment = True
                paths.extend(experiments[k]['files'])
                # write all to tmp.red.yml
                with open('.dvc_cc/tmp.red.yml',"w") as f:
                    print("batches:", file=f)
                    for path in paths:
                        with open(path,"r") as r:
                            print(r.read(), file=f)
                    with open('.dvc_cc/cc_config.yml',"r") as r:
                        print(r.read(), file=f)

                # execute faice
                output = subprocess.Popen(('faice exec .dvc_cc/tmp.red.yml').split(), stdout=subprocess.PIPE)
                cc_id = output.communicate()[0].decode().split()[-1]
                print('The experiment ID is: ' + cc_id)
                os.remove('.dvc_cc/tmp.red.yml')

                # write cc_id to cc_agency_experiments.yml
                experiments[k]['id'] = cc_id
        if start_an_experiment:
            with open('.dvc_cc/cc_agency_experiments.yml', 'w') as outfile:
                yaml.dump(experiments, outfile, default_flow_style=False)

        # push the ids
        subprocess.call(['git', 'add', '.dvc_cc/cc_agency_experiments.yml'])
        subprocess.call(['git', 'commit', '-m', '\'Update cc_agency_experiments.yml: ' + path + '\''])
        subprocess.call(['git', 'push'])
    else:
        print('Warning you did not define a job with dvc-cc run --no-exec. So there is no job to start.')

        
# TODO: CHECK IF A COMMIT WAS DONE BEFORE A NEW PROJEC

#
#### Improvements:
#
# - it should be possible to check if this repo already are in the yaml file and than it should not be pushed in the file!
# 
# - the BETTER solution is to use yaml see below:
#
#
# from ruamel.yaml import YAML
# yaml = YAML(typ='safe')
#
#red = {
#  'batches': [{
#     'inputs': {},
#     'outputs': {}
#   }]
#}
#
#with open(path, 'w') as f:
#  yaml.dump(red, f)
#



from collections import OrderedDict
from dvc_cc.version import VERSION
from dvc_cc.cli_modes import cli_modes

import os
import yaml
import requests
import keyring
from dvc.repo import Repo as DVCRepo
from git import Repo as GITRepo
from argparse import ArgumentParser
import datetime


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

def get_main_git_directory_path():
    gitrepo = GITRepo('.')
    git_path = gitrepo.common_dir.split('/.git')[0]
    return git_path

def show_all():
    with open(".dvc_cc/cc_config.yml", 'r') as stream:
        settings = yaml.safe_load(stream)
    if settings['container']['engine'] == 'nvidia-docker':
        print('%23s : %s' % ('num-of-gpus' , str(settings['container']['settings']['gpus']['count'])))
    else:
        print('%23s : %s' % ('num-of-gpus' , str(None)))

    print('%23s : %s GB' % ('ram' , str(int(settings['container']['settings']['ram']/1000))))
    print('%23s : %s' % ('docker-image' , str(settings['container']['settings']['image']['url'])))
    print('%23s : %s' % ('batch-concurrency-limit' , str(settings['execution']['settings']['batchConcurrencyLimit'])))
    print('%23s : %s' % ('retry-if-failed' , str(settings['execution']['settings']['retryIfFailed'])))
    print('%23s : %s' % ('engine' , str(settings['execution']['engine'])))
    print('%23s : %s' % ('engine-url' , str(settings['execution']['settings']['access']['url'])))

def setting_ram():
    parser = ArgumentParser(description='Show or set the RAM.')
    parser.add_argument('--set',help='Set the RAM in GB.', type=int, default=None)
    args = parser.parse_args()

    with open(".dvc_cc/cc_config.yml", 'r') as stream:
        settings = yaml.safe_load(stream)

    if args.set is None:
        print('%23s : %s GB' % ('ram' , str(int(settings['container']['settings']['ram']/1000))))
    else:
        settings['container']['settings']['ram'] = int(args.set * 1000)
        with open('.dvc_cc/cc_config.yml', 'w') as outfile:
            yaml.dump(settings, outfile)

def setting_docker_image():
    parser = ArgumentParser(description='Show or set the docker image.')
    parser.add_argument('--set',help='Set the docker image.', type=str, default=None)
    args = parser.parse_args()

    with open(".dvc_cc/cc_config.yml", 'r') as stream:
        settings = yaml.safe_load(stream)

    if args.set is None:
        print('%23s : %s' % ('docker-image' , str(settings['container']['settings']['image']['url'])))
    else:
        settings['container']['settings']['image']['url'] = args.set
        with open('.dvc_cc/cc_config.yml', 'w') as outfile:
            yaml.dump(settings, outfile)

def setting_batch_concurrency_limit():
    parser = ArgumentParser(description='Show or set the batch concurrency limit.')
    parser.add_argument('--set',help='Set the batch concurrency limit.', type=int, default=None)
    args = parser.parse_args()

    with open(".dvc_cc/cc_config.yml", 'r') as stream:
        settings = yaml.safe_load(stream)

    if args.set is None:
        print('%23s : %s' % ('batch-concurrency-limit' , str(settings['execution']['settings']['batchConcurrencyLimit'])))
    else:
        settings['execution']['settings']['batchConcurrencyLimit'] = args.set
        with open('.dvc_cc/cc_config.yml', 'w') as outfile:
            yaml.dump(settings, outfile)

def setting_engine():
    parser = ArgumentParser(description='Show or set the engine.')
    parser.add_argument('--set',help='Set the engine.', type=str, default=None)
    args = parser.parse_args()

    with open(".dvc_cc/cc_config.yml", 'r') as stream:
        settings = yaml.safe_load(stream)

    if args.set is None:
        print('%23s : %s' % ('engine' , str(settings['execution']['engine'])))
    else:
        settings['execution']['engine'] = args.set
        with open('.dvc_cc/cc_config.yml', 'w') as outfile:
            yaml.dump(settings, outfile)

def setting_engine_url():
    parser = ArgumentParser(description='Show or set the engine URL.')
    parser.add_argument('--set',help='Set the engine URL.', type=str, default=None)
    args = parser.parse_args()

    with open(".dvc_cc/cc_config.yml", 'r') as stream:
        settings = yaml.safe_load(stream)

    if args.set is None:
        print('%23s : %s' % ('engine-url' , str(settings['execution']['settings']['access']['url'])))
    else:
        settings['execution']['settings']['access']['url'] = args.set
        with open('.dvc_cc/cc_config.yml', 'w') as outfile:
            yaml.dump(settings, outfile)


def setting_retry_if_failed():
    parser = ArgumentParser(description='Show or set the setting: Retry if failed.')
    parser.add_argument('--set',help='Set the setting: Retry if failed.', type=bool, default=None)
    args = parser.parse_args()

    with open(".dvc_cc/cc_config.yml", 'r') as stream:
        settings = yaml.safe_load(stream)

    if args.set is None:
        print('%23s : %s' % ('retry-if-failed' , str(settings['execution']['settings']['retryIfFailed'])))
    else:
        settings['execution']['settings']['retryIfFailed'] = args.set
        with open('.dvc_cc/cc_config.yml', 'w') as outfile:
            yaml.dump(settings, outfile)

def setting_num_of_gpus():
    parser = ArgumentParser(description='Show or set the number of GPUs.')
    parser.add_argument('--set',help='Set the number of GPUs.', type=int, default=None)
    args = parser.parse_args()

    with open(".dvc_cc/cc_config.yml", 'r') as stream:
        settings = yaml.safe_load(stream)

    if args.set is None:
        if settings['container']['engine'] == 'nvidia-docker':
            print('%23s : %s' % ('num-of-gpus' , str(settings['container']['settings']['gpus']['count'])))
        else:
            print('%23s : %s' % ('num-of-gpus' , str(None)))
    else:
        if args.set <= 0:
            settings['container']['engine'] = 'docker'
            if 'gpus' in settings['container']['settings']:            
                settings['container']['settings']['gpus'].pop('count',None)
                settings['container']['settings'].pop('gpus',None)
        else:
            settings['container']['engine'] = 'nvidia-docker'
            settings['container']['settings']['gpus'] = {'count':args.set}
        with open('.dvc_cc/cc_config.yml', 'w') as outfile:
            yaml.dump(settings, outfile)



SCRIPT_NAME = 'dvc-cc setting'
TITLE = 'tools'
DESCRIPTION = 'DVC-CC (C) 2019  Jonas Annuscheit. This software is distributed under the AGPL-3.0 LICENSE.'
MODES = OrderedDict([
    ('all', {'main': show_all, 'description': 'Show all setting information.'}),
    ('ram', {'main': setting_ram, 'description': ''}),
    ('docker-image', {'main': setting_docker_image, 'description': 'Show or set the docker image.'}),
    ('batch-concurrency-limit', {'main': setting_batch_concurrency_limit, 'description': 'Show or set the batch concurrency limit.'}),
    ('engine', {'main': setting_engine, 'description': 'Show or set the engine.'}),
    ('engine-url', {'main': setting_engine_url, 'description': 'Show or set the engine URL.'}),
    ('retry-if-failed', {'main': setting_retry_if_failed, 'description': 'Show or set the setting: Retry if failed.'}),
    ('num-of-gpus', {'main': setting_num_of_gpus, 'description': 'Show or set the number of GPUs.'}),
])


def main():
    cli_modes(SCRIPT_NAME, TITLE, DESCRIPTION, MODES, VERSION)



from argparse import ArgumentParser
import subprocess
from subprocess import check_output
import os
from dvc_cc.bcolors import *


SCRIPT_NAME = 'dvc-cc init'
TITLE = 'tools'
DESCRIPTION = 'Scripts to initial a dvc-cc repository. It throws an exception, if the current project is not a git repository.'

from dvc.repo import Repo as DVCRepo
from git import Repo as GITRepo
 
def get_main_git_directory_path():
    gitrepo = GITRepo('.')
    git_path = gitrepo.common_dir.split('/.git')[0]
    return git_path

def get_gitinformation():
    # TODO: use the intern python-git for this.
    out = check_output(["git", "config", "--get", "remote.origin.url"]).decode("utf8")
    if out.startswith('https://'):
        _,_, gitrepo,gitowner,gitname = out.split('/')
    else:
        gitrepo = out[4:out.find(':')]
        gitowner = out[out.find(':')+1:out.find('/')]
        gitname = out[out.find('/')+1:]

    gitname = gitname[:gitname.find('.git')+4]
    return gitrepo,gitowner,gitname

def main():
    parser = ArgumentParser(description=DESCRIPTION)
    parser.add_argument('--not-interactive', help='If this parameter is set, it will not ask the user to set the values. All values will set by default values.',default=False, action='store_true')
    args = parser.parse_args()
    gitrepo,gitowner,gitname = get_gitinformation()

    if not args.not_interactive:
        print('These settings refer to the required hardware resources in the cluster.')
        print('If you do not set an argument it will take the default values.')

        print()
        print('Please enter the number of GPUs that you want on the cluster. Hint: In the most Deep Learning '
              'scripts, you want to use 1 GPU in the docke container.')
        num_of_gpus = None
        while num_of_gpus is None:
            num_of_gpus = input(bcolors.OKBLUE+'\tNumber of GPUs'+bcolors.ENDC+' (default 0): ')
            if num_of_gpus == '':
                num_of_gpus = 0
            elif num_of_gpus.isdigit():
                num_of_gpus = int(num_of_gpus)
            else:
                print(bcolors.FAIL + '\tWarning: Did not understand your answer. Please use integer values i.e. 0,1,2,3,...' + bcolors.ENDC)
                num_of_gpus = None

        print()
        print('Please enter the RAM that you want on the cluster.')
        ram = None
        while ram is None:
            ram = input(bcolors.OKBLUE+'\tRAM in GB'+bcolors.ENDC+' (default 20): ')
            if ram == '':
                ram = 20000 # 20 GB
            elif ram.isdigit():
                ram = int(ram)*1000
            else:
                print(bcolors.FAIL + '\tWarning: Did not understand your answer. Please use integer values i.e. 10,100,...'+bcolors.ENDC)
                ram = None

        print()
        print('Please enter the Docker Image in which your script get executed at the cluster.')
        print('   You can choose from the following:')
        print('     - "tf2", if you want to work with TensorFlow 2.0.')
        print('     - "tf1", if you want to work with TensorFlow 1.4.')
        print('     - "torch", if you want to work with PyTorch 1.2.')
        print('     - "large", if you want to work with PyTorch 1.2 or/and TensorFlow 2.0.')
        print('     - "basic", if you want install it by yourself via the Requirements.txt.')
        print('   You can also enter a URL to your own Docker Image.')
        print('   If you need more informations take a look at the following site: https://bit.ly/2mgbiVK')
        docker_image = input(bcolors.OKBLUE+'\tDocker Image'+bcolors.ENDC+' (default: "large"): ')
        if docker_image == '' or docker_image.lower() == 'large':
            docker_image = 'docker.io/deepprojects/dvc-cc_large:dev'
            docker_image_needs_credentials = False
        elif docker_image.lower() == 'tf2':
            docker_image = 'docker.io/deepprojects/dvc-cc_tensorflow:2.0.0rc'
            docker_image_needs_credentials = False
        elif docker_image.lower() == 'tf1':
            docker_image = 'docker.io/deepprojects/dvc-cc_tensorflow:1.14'
            docker_image_needs_credentials = False
        elif docker_image.lower() == 'torch':
            docker_image = 'docker.io/deepprojects/dvc-cc_pytorch:1.2'
            docker_image_needs_credentials = False
        elif docker_image.lower() == 'basic':
            docker_image = 'docker.io/deepprojects/dvc-cc_basic:dev'
            docker_image_needs_credentials = False
        else:
            docker_image_needs_credentials = None
            while docker_image_needs_credentials is None:
                docker_image_needs_credentials = input('\tDoes this docker image needs '
                                                       ''+bcolors.OKBLUE+'credentials'+bcolors.ENDC+'? [y,n]:')
                if docker_image_needs_credentials.lower().startswith('y'):
                    docker_image_needs_credentials = True
                elif docker_image_needs_credentials.lower().startswith('n'):
                    docker_image_needs_credentials = False
                else:
                    print(bcolors.FAIL+'\tWarning: Did not understand your answer. Please use y or n.'+bcolors.ENDC)
                    docker_image_needs_credentials = None
        print('You will use the Docker Image: '+ docker_image)
        print()
        batch_concurrency_limit = None
        print('The batch concurrency limit describes how many jobs you can start in parallel.')
        print('You can lower the number to 1, if you do not want the jobs from one experiment runs in parallel.')
        while batch_concurrency_limit is None:
            batch_concurrency_limit = input(bcolors.OKBLUE+'\tBatch concurrency limit'+bcolors.ENDC+' (default 12): ')
            if batch_concurrency_limit == '':
                batch_concurrency_limit = 12
            elif batch_concurrency_limit.isdigit():
                batch_concurrency_limit = int(batch_concurrency_limit)
            else:
                print(bcolors.FAIL+'\tWarning: Did not understand your answer. Please use integer values i.e. 1,4,12,...'+bcolors.ENDC)
                batch_concurrency_limit = None

        print()
        print('The name of the engine you want to use. This describes the cluster that you want to use.')
        print('At the HTW we have the engines "dt", "cc" and "cctest".')

        engine = input('\tThe '+bcolors.OKBLUE+'engine'+bcolors.ENDC+' you want to use (default: dt): ')
        if engine == '' or engine == 'dt':
            engine = 'ccagency'
            engine_url = 'https://agency.f4.htw-berlin.de/dt'
        elif engine == 'cc':
            engine = 'ccagency'
            engine_url = 'https://agency.f4.htw-berlin.de/cc'
        elif engine == 'cctest':
            engine = 'ccagency'
            engine_url = 'https://agency.f4.htw-berlin.de/cctest'
        else:
            print('\tThis engine is unknown. Please specify the engine-url:')
            engine_url = input('The ' + bcolors.OKBLUE + 'engine-url' + bcolors.ENDC + ' you want to use: ')

        print('You will use the engine "' +engine+'" with the url "'+engine_url+'".')

        print()
        print('All large files created by your script and defined as output files by DVC are stored on the DVC server.')
        print('At the HTW we have the storage server "dt1" and "avocado01".')
        dvc_remote_server = input('\tThe remote '+bcolors.OKBLUE+'DVC server'+bcolors.ENDC+' that you want use ('
                                                                                          'default: dt1): ')
        if dvc_remote_server == '' or dvc_remote_server.lower() == 'dt' or dvc_remote_server.lower() == 'dt1':
            dvc_remote_server = 'dt1.f4.htw-berlin.de'
        elif dvc_remote_server.lower() == 'avocado' or dvc_remote_server.lower() == 'avocado01':
            dvc_remote_server = 'avocado01.f4.htw-berlin.de'
        print('You will use the following DVC server "' + dvc_remote_server + '".')


        print()
        print('Here you can enter the folder where you want to store the DVC files on the DVC Storage Server.')
        if engine_url == 'https://agency.f4.htw-berlin.de/cc' or engine_url == \
                'https://agency.f4.htw-berlin.de/cctest':
            dvc_folder_default_value = '/data/ldap/Data-Version-Control-Cache/' + gitrepo + '/' + gitowner + '/' + \
                                       gitname
        else:
            dvc_folder_default_value = gitrepo + '/' + gitowner + '/' + gitname
        dvc_remote_path = input('\tThe remote '+bcolors.OKBLUE+'DVC folder'+bcolors.ENDC+' that you want use ('
                                                                                         'default: '+dvc_folder_default_value+'): ')
        if dvc_remote_path == '':
            dvc_remote_path = dvc_folder_default_value

        print()
        print('The username with that you can access the DVC storage server "'+dvc_remote_server+'".')
        dvc_remote_user = input('\tThe '+bcolors.OKBLUE+'username'+bcolors.ENDC+' for the remote DVC folder: ')
        if dvc_remote_user == '':
            dvc_remote_user = input('Do you really want to use the connection to the remote dvc folder without credentials? [n,y]')
            if not dvc_remote_user.lower().startswith('y'):
                dvc_remote_user = input('The username for the remote DVC folder: ')
        print()
    else:
        # set default values
        num_of_gpus = 0 ##
        ram = 131072
        docker_image = 'docker.io/deepprojects/dvc-cc_large:dev'
        docker_image_needs_credentials = False
        batch_concurrency_limit = 12
        engine = 'ccagency'
        engine_url = 'https://agency.f4.htw-berlin.de/dt'
        dvc_remote_server = 'dt1.f4.htw-berlin.de'
        dvc_remote_path = gitrepo + '/' + gitowner + '/' + gitname
        dvc_remote_user = ''

    # Change the directory to the main git directory.
    os.chdir(get_main_git_directory_path())
    
    gitrepo = GITRepo('.')
    try:
        dvcrepo = DVCRepo('.')

        #TODO: this can be removed!?
        if not os.path.exists('.dvc'):
            dvcrepo.init()
    except:
        subprocess.call(['dvc', 'init'])
        dvcrepo = DVCRepo('.')

    # set remote dvc connection
    if dvc_remote_user == '':
        subprocess.call(
            ['dvc', 'remote', 'add', '--force', '-d', 'dvc_connection', 'ssh://' + dvc_remote_server + ':' + dvc_remote_path])
        subprocess.call(['dvc', 'remote', 'modify', 'dvc_connection', 'ask_password', 'false'])
    else:
        subprocess.call(['dvc', 'remote', 'add', '--force', '-d', 'dvc_connection',
                         'ssh://' + dvc_remote_user + '@' + dvc_remote_server + ':' + dvc_remote_path])
        subprocess.call(['dvc', 'remote', 'modify', 'dvc_connection', 'ask_password', 'true'])
    # test remote connection
    subprocess.call(['dvc','push'])

    try:
        subprocess.call(['ssh', dvc_remote_user + '@' + dvc_remote_server, "mkdir -p "+dvc_remote_path+" ; chmod 774 "+dvc_remote_path+" ; setfacl -d -m u::rwX,g::rwX,o::- "+dvc_remote_path])
    except:
        print(bcolors.WARNING+'Warning: Currently acl is not installed on the server! You will maybe have problems by sharing the same remote dvc folder!'+bcolors.ENDC)


    # create the main folder of the dvc_cc software package.
    if not os.path.exists('.dvc_cc'):
        os.mkdir('.dvc_cc')
    
    # create the config file.    
    if os.path.exists('.dvc_cc/cc_config.yml'):
        os.remove('.dvc_cc/cc_config.yml')

    create_cc_config_file(num_of_gpus,ram,docker_image, docker_image_needs_credentials, batch_concurrency_limit, engine, engine_url)
    subprocess.call(['git', 'add', '.dvc_cc/cc_config.yml'])
    #TODO: CREATE THE SAMPLE PROJECTS !!!


def create_cc_config_file(num_of_gpus,ram,docker_image, docker_image_needs_credentials, batch_concurrency_limit, engine, engine_url):
    with open('.dvc_cc/cc_config.yml',"w") as f:
        print("cli:", file=f)
        print("  baseCommand: [dvc-cc-agent]", file=f)
        print("  class: CommandLineTool", file=f)
        print("  cwlVersion: v1.0", file=f)
        print("  doc: some descriptions of the package...", file=f)
        print("  inputs:", file=f)
        print("    git_authentication_json:", file=f)
        print("      doc: 'A path to json file which contains the git authentication. This should include the keys: username. email and password.'", file=f)
        print("      inputBinding: {position: 0}", file=f)
        print("      type: File", file=f)
        print("    git_path_to_working_repository:", file=f)
        print("      doc: 'The git working directory. With this you can specify what the main git root is.'", file=f)
        print("      inputBinding: {position: 1}", file=f)
        print("      type: string", file=f)
        print("    git_working_repository_owner:", file=f)
        print("      doc: 'The name of the owner of the git repository which you want to execute.'", file=f)
        print("      inputBinding: {position: 2}", file=f)
        print("      type: string", file=f)
        print("    git_working_repository_name:", file=f)
        print("      doc: 'The git repository name.'", file=f)
        print("      inputBinding: {position: 3}", file=f)
        print("      type: string", file=f)
        print("    git_name_of_branch:", file=f)
        print("      doc: 'The source code jumps to this here defined git tag (with git checkout) and execute dvc repro there.'", file=f)
        print("      inputBinding: {position: 4}", file=f)
        print("      type: string", file=f)
        print("    dvc_authentication_json:", file=f)
        print("      doc: 'A path to json file which contains the dvc authentication. This should include the keys: username and password.'", file=f)
        print("      inputBinding: {position: 5}", file=f)
        print("      type: File", file=f)
        print("    dvc_servername:", file=f)
        print("      doc: 'The servername of the dvc directory.'", file=f)
        print("      inputBinding: {position: 6}", file=f)
        print("      type: string", file=f)
        print("    dvc_path_to_working_repository:", file=f)
        print("      doc: 'The directory that is used for the dvc script.'", file=f)
        print("      inputBinding: {position: 7}", file=f)
        print("      type: string", file=f)
        print("    dvc_data_dir:", file=f)
        print("      doc: 'This is optional parameter. Here you can specify a sshfs folder for the \"data\" folder.'", file=f)
        print("      inputBinding: {prefix: --data_dir}", file=f)
        print("      type: Directory?", file=f)
        print("    dvc_remote_directory_sshfs:", file=f)
        print("      doc: 'A SSHFS connection to stream the output of DVC REPRO -P.'", file=f)
        print("      inputBinding: {prefix: --dvc_remote_directory_sshfs}", file=f)
        print("      type: Directory?", file=f)
        print("    dvc_file_to_execute:", file=f)
        print("      doc: 'This is optional parameter. If this parameter is given it will run \"dvc repro DVC_FILE_TO_EXECUTE\". Is this parameter is not set it will run \"dvc repro -P\"'", file=f)
        print("      inputBinding: {prefix: --dvc_file_to_execute}", file=f)
        print("      type: string?", file=f)

        print("    live_output_files:", file=f)
        print("      doc: 'Comma separated string list of files that should be included to the live output for example: tensorboard,output.json This could track a tensorboard folder and a output.json file.'", file=f)
        print("      inputBinding: {prefix: --live_output_files}", file=f)
        print("      type: string?", file=f)
        print("    live_output_update_frequence:", file=f)
        print("      doc: 'The update frequence of the live output in seconds.'", file=f)
        print("      inputBinding: {prefix: --live_output_update_frequence}", file=f)
        print("      type: int?", file=f)

        print("  outputs: {}", file=f)
        print("container:", file=f)
        print("  engine: docker", file=f)
        print("  settings:", file=f)
        if num_of_gpus > 0:
            print("    gpus:", file=f)
            print("      count: "+str(num_of_gpus), file=f)
            print("      vendor: \"nvidia\"", file=f)

        # TODO: ASK FOR THIS!
        print("    image:", file=f)
        print("      url: '"+docker_image+"'", file=f)
        if docker_image_needs_credentials:
            print("      auth:", file=f)
            print("        password: '{{docker_image_password}}'", file=f)
            print("        username: '{{docker_image_username}}'", file=f)
        print("    ram: "+str(ram), file=f)
        print("execution:", file=f)
        print("  engine: " + engine, file=f)
        print("  settings:", file=f)
        print("    access:", file=f)
        print("      auth: {password: '{{agency_password}}', username: '{{agency_username}}'}", file=f)    
        print("      url: " + engine_url, file=f)
        print("    batchConcurrencyLimit: "+str(batch_concurrency_limit), file=f)
        print("    retryIfFailed: false", file=f)
        print("redVersion: '8'", file=f)
        
        
